#version 450 core

in vec3 pixelPosition;

layout(std140, binding = 0) uniform VP{
    mat4 view;
    mat4 projection;
};

uniform float radius;
uniform vec3 center;

uniform samplerCube skybox;

out vec4 billboardColor;

void main(){

    //Constructing Ray through pixelPosition
    vec3 cameraPosition = vec3(0,0,0);
    vec3 vectorV = normalize(pixelPosition - cameraPosition);

    //Vector from cameraPosition to sphereOrigin
    vec3 sphereOrigin = (view* vec4(center,1)).xyz;
    vec3 vectorCO = cameraPosition - sphereOrigin;

    //Komponenten der Quadratischen Gleichung
    float b = 2* dot(vectorCO , vectorV);
    float c = dot(vectorCO, vectorCO) - (radius * radius);
    float underSqrt = (b/2)*(b/2) - c;

    float t;
    if (underSqrt < 0.0){
        discard;
    }
    else{
        float t1 = ((-(b/2)) - sqrt(underSqrt));
        float t2 = ((-(b/2)) + sqrt(underSqrt));
        t = min(t1,t2);

        //Schnittpunkt mit der Kugel
        vec3 P = cameraPosition + t * vectorV;

        //gl_FragDepth = P.z;

        //Normale der Kugel am Schnittpunkt in view space
        vec3 PNormal = normalize(P - sphereOrigin);

//diffuse Beleuchtung der Kugel

        //Vector vom Schnittpunkt zur Lichtquelle
        vec3 PLight = - vectorV;
        //Cosinus des Winkels zwischen der Normale und dem Vektor ZUM Licht
        //zur berechnung des diffusen Lichtes
        float cos_phi = clamp( dot( PNormal,PLight ), 0, 1);

        //Berechnung des diffusen Lichtes, t ist quasi die "Distanz" zur Kugel
        vec3 diffuse = vec3(cos_phi / (1 +(t*0.01)),
                            cos_phi / (1 +(t*0.01)),
                            cos_phi / (1 +(t*0.01)));
        //Cel-Shading
/**
        int csLevel = 3;
        diffuse.x = (round(diffuse.x * csLevel)) / csLevel;
        diffuse.y = (round(diffuse.y * csLevel)) / csLevel;
        diffuse.z = (round(diffuse.z * csLevel)) / csLevel;


        float r = diffuse.r * 0.2;
        float g = diffuse.g * 1.0;
        float b = diffuse.b * 0.5;
        billboardColor = vec4(r,g,b,1);
*/

//Reflektion der Skybox in der Kugel

        vec3 PReflect = reflect(P, PNormal);
        billboardColor = texture(skybox,mat3(inverse(view)) * PReflect);

        //choose his to just mirror the skybox without taking camera position into account
        //can be use for debugging
        //billboardColor = texture(skybox, mat3(inverse(view))*PNormal);



//Refraktion der Skybox in der Kugel
/**
        float ratioIn = 1.00 / 1.52 ;
        float ratioOut = 1.52 / 1.00;
        vec3 PRefractIn = normalize(refract(normalize(P), PNormal, ratioIn));

        //Komponenten der Quadratischen Gleichung
        vec3 vectorPO = P - sphereOrigin;
        float b = 2* dot(vectorPO , normalize(PRefractIn));
        float c = dot(vectorPO, vectorPO) - (radius * radius);
        float underSqrt = (b/2)*(b/2) - c;

        float k1 = ((-(b/2)) - sqrt(underSqrt));
        float k2 = ((-(b/2)) + sqrt(underSqrt));
        float k = max(k1,k2);

        vec3 P2 = P + k* PRefractIn;
        vec3 P2Normal = normalize(P2 - sphereOrigin);

        vec3 PRefractOut =  refract(normalize(P2 - P), -P2Normal, ratioOut);

        billboardColor =  vec4(vec3(texture(skybox,mat3(inverse(view)) * PRefractOut)),0.5f);
        */


    }

}

