#version 450 core

layout (triangles) in;
//the geometry shader can handle a maximum of 78 output vertices in this configuration
//for every light source, we have to create 18 out vertices
//thus, we can handle a maximum of 4 lights with this shader, 4*18 = 72

// !!!! IMPORTANT TODO !!!!
// This number (68) has been temporarily reduced (from 72) for the prototype to make the alpha blending work!
layout (triangle_strip, max_vertices=68) out;

uniform mat4 lightSpaceMatrices[55];
uniform vec3 lightPositions[11];
uniform float farPlanes[11];
uniform int numLayers;

uniform bool hasWindTexture;
uniform sampler2D texture_wind1;
uniform mat4 model;
uniform float windCycle;
uniform bool windEnable;

in VertexData
{
	vec3 positionOS;
	vec2 texCoords;
	vec3 normalOS;
	vec3 tangentOS;
	vec3 bitangentOS;
} gs_in[];

out vec4 worldPosFrag;
out vec3 lightPos;
out float farPlane;

out vec2 geoTexCoords;


vec4 applyWind(vec4 posWS, vec2 texCoord, vec3 normalWS_) {
	if (!hasWindTexture || !windEnable)
		return posWS;

	float modelWindFactor = 1.0f - textureLod(texture_wind1, texCoord, 0.0f).r;
	vec3 windVector = vec3(
		0,
		0,
		2.5 * pow(modelWindFactor, 2) * abs(sin(3.1415926 * modelWindFactor + windCycle))
	);

	return vec4(posWS.xyz + windVector * abs(normalWS_), 1.0f);
}

void main()
{
	mat3 invTranspModel = mat3(transpose(inverse(model)));
	vec3 normalsWS[3];
	normalsWS[0] = normalize(invTranspModel * gs_in[0].normalOS);
	normalsWS[1] = normalize(invTranspModel * gs_in[1].normalOS);
	normalsWS[2] = normalize(invTranspModel * gs_in[2].normalOS);
	
	vec4 positionsWS[3];
	positionsWS[0] = applyWind(model * vec4(gs_in[0].positionOS, 1.0f), gs_in[0].texCoords, normalsWS[0]);
	positionsWS[1] = applyWind(model * vec4(gs_in[1].positionOS, 1.0f), gs_in[1].texCoords, normalsWS[1]);
	positionsWS[2] = applyWind(model * vec4(gs_in[2].positionOS, 1.0f), gs_in[2].texCoords, normalsWS[2]);
	
	
	for (int layer = 0; layer < numLayers; layer++)
	{
		for(int face = 0; face < 6; ++face)
		{
			for(int i = 0; i < 3; ++i)
			{
				int layerFace = face + (layer*6);
				gl_Layer = layerFace;
				worldPosFrag = positionsWS[i];
				lightPos = lightPositions[layer];
				farPlane = farPlanes[layer];
				gl_Position = lightSpaceMatrices[layerFace] * worldPosFrag;
				geoTexCoords = gs_in[i].texCoords;
				EmitVertex();
			}    
			EndPrimitive();
		}
	}
} 