#version 450 core

in vec3 worldPosFrag;
in vec3 worldNormal;
in vec2 texCoords;

uniform vec3 cameraPos;
uniform float far_plane;

uniform sampler2D texture_diffuse1;
uniform sampler2D texture_specular1;

layout(std140, binding=1) uniform Lights{
	int numLights;
	vec4 position[11];
	vec3 ambient[11];
	vec3 diffuse[11];
	vec3 specular[11];
	vec3 attenuation[11];
	float lightFar[11];
};

out vec4 FragColor;


void main() {

	//Lighting calculations in world space

	vec3 result = vec3(0.0f, 0.0f, 0.0f);

	for(int i = 0;i < numLights;i++){
		
		//directional light
		if(position[i].w == 0){
			//AMBIENT
			vec3 amb = ambient[i] * texture(texture_diffuse1, texCoords).xyz;
			//DIFFUSE
			vec3 fragNormal = normalize(worldNormal);
			vec3 direction =  normalize(position[i].xyz);

			float cosine = max(dot(fragNormal, - direction), 0.0f);
			vec3 diff = (diffuse[i] * cosine) * texture(texture_diffuse1, texCoords).xyz;

			//SPECULAR use halfway vector for Blinn-Phong shading
			vec3 fragToLight = normalize(position[i].xyz - worldPosFrag);
			vec3 fragToEye =normalize(cameraPos - worldPosFrag);
			vec3 halfway = normalize(fragToLight + fragToEye);

			cosine = max(dot(fragNormal, halfway), 0.0f);
			vec3 spec = (specular[i] * pow(cosine, 20.0f)) * texture(texture_specular1, texCoords).xyz;

			result += (spec);
			}
		
		//point light
		if(position[i].w == 1){
			
			//AMBIENT
			vec3 amb = ambient[i] * texture(texture_diffuse1, texCoords).xyz;

			//DIFFUSE
			vec3 fragNormal = normalize(worldNormal);
			vec3 fragToLight =  normalize(position[i].xyz - worldPosFrag);
			float cosine = max(dot(fragNormal, fragToLight), 0.0f);
			vec3 diff = (diffuse[i] * cosine) * texture(texture_diffuse1, texCoords).xyz;

			//SPECULAR
			vec3 fragToEye =normalize(cameraPos - worldPosFrag);
			vec3 halfway = normalize(fragToLight + fragToEye);

			cosine = max(dot(fragNormal, halfway), 0.0f);
			vec3 spec = (specular[i] * pow(cosine, 20.0f)) * texture(texture_specular1, texCoords).xyz;

			//ATTENUATION
			float dist = length(position[i].xyz - worldPosFrag);
			float att = 1.0 / (attenuation[i][0] + attenuation[i][1]*dist + attenuation[i][2]* (dist*dist));

			result = (amb + diff + spec);
			}
		}

	//Depth information
    float lightDistance = length(worldPosFrag - cameraPos);
    
    // map to [0;1] range by dividing by far_plane
    lightDistance = lightDistance / far_plane;
    
    // write this as modified depth
    gl_FragDepth = lightDistance;

	FragColor = vec4(result, 1.0f);
}