#version 450 core

//If you declare a uniform that isn't used anywhere in your GLSL code the compiler will silently
//remove the variable from the compiled version which is the cause for several frustrating errors; keep this in mind!

layout (location = 0) in vec3 position;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 texcoords;
layout (location = 3) in vec3 tangent;
layout (location = 4) in vec3 bitangent;

layout(std140, binding = 0) uniform VP
{
    mat4 view;
    mat4 projection;
    vec4 waterClipPlane;
};

uniform mat4 viewProjection;
uniform mat4 model;
uniform mat3 normalMat;


out vec3 worldPosFrag;
out vec2 texCoords;
out mat3 TBN;

void main()
{
	vec4 positionWS = model * vec4(position, 1.0f);
    worldPosFrag = vec3(positionWS);
	gl_Position = viewProjection * positionWS;
    texCoords = texcoords;
    
    //Gram Schmidt because tangent vectors might get interpolated when calculated in model loading, resulting in non-orthogonal matrix
    //also, we just calculate the bitangent vector using cross product so we don't have to load it in;

    vec3 T = normalize(normalMat * tangent);
    vec3 N = normalize(normalMat * normal);
    vec3 B = normalize(normalMat * bitangent);

    //T = normalize(T - dot(T,N) * N);
    //vec3 B = cross(T, N);

    TBN = mat3(T,B,N);


    gl_ClipDistance[0] =  dot(vec4(worldPosFrag,1), waterClipPlane);


}