#version 450 core

#define VOXEL_GRID_SIZE_X 160
#define VOXEL_GRID_SIZE_Y 90
#define VOXEL_GRID_SIZE_Z 128

in vec3 cubemapDir;

uniform samplerCube cubemap;
uniform sampler3D volFogVoxelGrid;
uniform bool volFogEnable;

uniform int FB_WIDTH;
uniform int FB_HEIGHT;

layout(std140, binding = 2) uniform UserInput
{
    ivec4 volFogAct_blueNoise_filterAct;
};

out vec4 FragColor;

float gauss(int offset, float sigma) {
    return exp(-(offset * offset) / (2.0 * sigma * sigma)) / (sqrt(2.0 * 3.14159265) * sigma);
}

vec4 gauss_1d(float sigma, vec3 uvw, int axis, int axisGridSize) {
    float weightSum = 0.0f;
    vec4 result = vec4(0.0f, 0.0f, 0.0f, 0.0f);
    for (int i = -2; i < 3; i++) {
        float newAxisValue = (uvw[axis] * axisGridSize + i) / axisGridSize;
		
		// Ensure we don't go out of bounds
		if (newAxisValue < 0.0f)
            newAxisValue = 0.0f;
        else if (newAxisValue > 1.0f)
            newAxisValue = 1.0f;

        uvw[axis] = newAxisValue;

        float w = gauss(i, sigma);
        weightSum += w;
        result += w * textureLod(volFogVoxelGrid, uvw, 0.0f);
    }
    return result / weightSum;
}


vec4 blur(vec3 uvw) {
    float sigma = 1;
    
	// Blur in each direction seperately -> way faster than applying a 3D kernel.
    vec4 xResult = gauss_1d(sigma, uvw, 0, VOXEL_GRID_SIZE_X);
    vec4 yResult = gauss_1d(sigma, uvw, 1, VOXEL_GRID_SIZE_Y);
    vec4 zResult = gauss_1d(sigma, uvw, 2, VOXEL_GRID_SIZE_Z);

    return (xResult + yResult + zResult) / 3.0f;
}

vec3 applyVolumetricFog(vec3 color) {
	vec3 uvw = vec3(float(gl_FragCoord.x)/(FB_WIDTH - 1), float(gl_FragCoord.y)/(FB_HEIGHT - 1), 1.0f);
    vec4 volFogLight = volFogAct_blueNoise_filterAct.z == 1 ? blur(uvw) : textureLod(volFogVoxelGrid, uvw, 0.0f);	

	return color * volFogLight.a + volFogLight.rgb;
	//return color * max(volFogLight.a, 0.1f) + volFogLight.rgb * 0.1f;
	//return color * max(volFogLight.a, 0.7f) + volFogLight.rgb * 0.2f;
}

void main(){
	if (volFogAct_blueNoise_filterAct.x == 1 && volFogEnable) {
		vec4 envColor = texture(cubemap, cubemapDir);
		FragColor = vec4(applyVolumetricFog(vec3(envColor.rgb)), 1.0);
	}
	else
		FragColor = texture(cubemap, cubemapDir);

}