#version 450 core
// -------------------------------------------------------


struct PatchData
{
	vec3 b300;
	vec3 b030;
	vec3 b003;
	vec3 b210;
	vec3 b120;
	vec3 b021;
	vec3 b012;
	vec3 b102;
	vec3 b201;
	vec3 b111;

	vec3 n200;
	vec3 n020;
	vec3 n002;
	vec3 n110;
	vec3 n011;
	vec3 n101;
};


layout(triangles, equal_spacing, cw) in;

in TescTeseData
{
	vec2 texCoords;
	vec3 tangentOS;
	vec3 bitangentOS;
} tc_in[];

layout (location = 4) patch in PatchData pd;

out VertexData
{
	vec3 positionOS;
	vec2 texCoords;
	vec3 normalOS;
	vec3 tangentOS;
	vec3 bitangentOS;
} te_out;


vec3 calcPosition()
{
	vec3 b300 = pd.b300;
	vec3 b030 = pd.b030;
	vec3 b003 = pd.b003;
	vec3 b210 = pd.b210;
	vec3 b120 = pd.b120;
	vec3 b021 = pd.b021;
	vec3 b012 = pd.b012;
	vec3 b102 = pd.b102;
	vec3 b201 = pd.b201;
	vec3 b111 = pd.b111;

	float u = gl_TessCoord.x;
	float v = gl_TessCoord.y;
	float w = gl_TessCoord.z;

	float u2 = pow(u, 2);
	float v2 = pow(v, 2);
	float w2 = pow(w, 2);


	return b300 * pow(u, 3) + b030 * pow(v, 3) + b003 * pow(w, 3) +
	b210 * 3 * u2 * v + b120 * 3 * u * v2 + b201 * 3 * u2 * w +
	b021 * 3 * v2 * w + b102 * 3 * u * w2 + b012 * 3 * v * w2 +
	b111 * 6 * w * u * v;
	//return b300 * u + b030 * v + b003 * w;
}

vec3 calcNormal()
{
	vec3 n200 = pd.n200;
	vec3 n020 = pd.n020;
	vec3 n002 = pd.n002;
	vec3 n011 = pd.n011;
	vec3 n101 = pd.n101;
	vec3 n110 = pd.n110;

	float u = gl_TessCoord.x;
	float v = gl_TessCoord.y;
	float w = gl_TessCoord.z;

	//vec3 n = n200 * u + n020 * v + n002 * w;
	vec3 n = n200 * pow(u, 2) + n020 * pow(v, 2) + n002 * pow(w, 2) +
	n110 * u * v + n011 * v * w + n101 * u * w;

	return normalize(n);
}

void main()
{
	vec2 vertexTexCoord         =           gl_TessCoord.x * tc_in[0].texCoords   + gl_TessCoord.y * tc_in[1].texCoords   + gl_TessCoord.z * tc_in[2].texCoords;
	vec3 vertexTangentOS        = normalize(gl_TessCoord.x * tc_in[0].tangentOS   + gl_TessCoord.y * tc_in[1].tangentOS   + gl_TessCoord.z * tc_in[2].tangentOS);
	vec3 vertexBitangentOS      = normalize(gl_TessCoord.x * tc_in[0].bitangentOS + gl_TessCoord.y * tc_in[1].bitangentOS + gl_TessCoord.z * tc_in[2].bitangentOS);

	vec4 vertexPositionOS       = vec4(calcPosition(), 1.0);
	vec3 vertexNormalOS         = calcNormal();
		
	te_out.positionOS  = vertexPositionOS.xyz;
	te_out.texCoords   = vertexTexCoord;
	te_out.normalOS    = vertexNormalOS;
	te_out.tangentOS   = vertexTangentOS;
	te_out.bitangentOS = vertexBitangentOS;

	gl_Position = vertexPositionOS;
}
