#version 450 core

//If you declare a uniform that isn't used anywhere in your GLSL code the compiler will silently
//remove the variable from the compiled version which is the cause for several frustrating errors; keep this in mind!

layout (location = 0) in vec3 position;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 texcoords;

layout(std140, binding = 0) uniform VP
{
    mat4 view;
    mat4 projection;
    vec4 waterClipPlane;
};

uniform mat4 model;
uniform mat3 normalMat;

out vec3 worldPosFrag;
out vec3 worldNormal;
out vec2 texCoords;
out vec4 clipSpace;

void main()
{
    clipSpace = projection * view * model *vec4(position, 1.0f);
    gl_Position = clipSpace;
    worldPosFrag = vec3(model *vec4(position, 1.0f));    
    worldNormal = normalize(normalMat * normal);
    texCoords = texcoords;
    gl_ClipDistance[0] =  dot(vec4(worldPosFrag,1), waterClipPlane);


}