#version 450 core

layout (triangles) in;
layout (triangle_strip, max_vertices = 3) out;


uniform bool hasWindTexture;
uniform sampler2D texture_wind1;
uniform bool windEnable;

out vec3 worldPosFrag;
out vec2 texCoords;
out mat3 TBN;

in VertexData
{
	vec3 positionOS;
	vec2 texCoords;
	vec3 normalOS;
	vec3 tangentOS;
	vec3 bitangentOS;
} gs_in[];

out vec3 positionOS; 

layout(std140, binding = 0) uniform VP
{
    mat4 view;
    mat4 projection;
    vec4 waterClipPlane;
};

uniform float windCycle;

uniform mat4 model;
uniform mat4 modelMatrix;
uniform mat4 viewProjectionMatrix;


vec4 applyWind(vec4 pos, vec2 texCoord, vec3 normalWS) {
	if (!hasWindTexture || !windEnable)
		return pos;
	float modelWindFactor = 1.0f - textureLod(texture_wind1, texCoord, 0.0f).r;

	vec3 windVector = vec3(
		0,
		0,
		2.5 * pow(modelWindFactor, 2) * abs(sin(3.1415926 * modelWindFactor + windCycle))
	);

	return vec4(pos.xyz + windVector * abs(normalWS), 1.0f);
}

void recalculateTBN(mat3 invTanspModel, vec3 pos1, vec3 pos2, vec3 pos3, vec2 texCoords1, vec2 texCoords2, vec2 texCoords3) {
	vec3 edge1 = pos2 - pos1;
	vec3 edge2 = pos3 - pos1;
	vec3 normal = normalize(cross(edge1, edge2));

	vec2 deltaUV1 = texCoords2 - texCoords1;
	vec2 deltaUV2 = texCoords3 - texCoords1;
	float r = 1.0f / (deltaUV1.x * deltaUV2.y - deltaUV1.y * deltaUV2.x);
	vec3 tangent = normalize((edge1 * deltaUV2.y - edge2 * deltaUV1.y) * r);
	
	TBN = mat3(
		tangent,
		normalize(cross(tangent, normal)),
		normal
	);
}


void main() {
	mat3 invTranspModel = mat3(transpose(inverse(model)));
	vec4 newPos0 = applyWind(model * vec4(gs_in[0].positionOS, 1.0f), gs_in[0].texCoords, invTranspModel * gs_in[0].normalOS);
	vec4 newPos1 = applyWind(model * vec4(gs_in[1].positionOS, 1.0f), gs_in[1].texCoords, invTranspModel * gs_in[1].normalOS);
	vec4 newPos2 = applyWind(model * vec4(gs_in[2].positionOS, 1.0f), gs_in[2].texCoords, invTranspModel * gs_in[2].normalOS);
	
	
	gl_Position	= projection * view * newPos0;
    texCoords	= gs_in[0].texCoords;
    worldPosFrag= newPos0.xyz;
	
	TBN = mat3(
		normalize(invTranspModel * gs_in[0].tangentOS),
		normalize(invTranspModel * gs_in[0].bitangentOS),
		normalize(invTranspModel * gs_in[0].normalOS)
	);
	//ToDo: Check if we need to recalculate the TBN matrix since we move the vertices...
	//TBN = gs_in[0].TBN0;
	//recalculateTBN(invTranspModel, newPos0.xyz, newPos1.xyz, newPos2.xyz, gs_in[0].texCoords, gs_in[1].texCoords, gs_in[2].texCoords);
    EmitVertex();

	
    gl_Position = projection * view * newPos1;
	texCoords	= gs_in[1].texCoords;
    worldPosFrag= newPos1.xyz;
    
	TBN = mat3(
		normalize(invTranspModel * gs_in[1].tangentOS),
		normalize(invTranspModel * gs_in[1].bitangentOS),
		normalize(invTranspModel * gs_in[1].normalOS)
	);
	//TBN = gs_in[1].TBN0;
	//recalculateTBN(invTranspModel, newPos1.xyz, newPos2.xyz, newPos0.xyz, gs_in[1].texCoords, gs_in[2].texCoords, gs_in[0].texCoords);
	EmitVertex();

	
    gl_Position = projection * view * newPos2;
	texCoords	= gs_in[2].texCoords;
    worldPosFrag= newPos2.xyz;
	TBN = mat3(
		normalize(invTranspModel * gs_in[0].tangentOS),
		normalize(invTranspModel * gs_in[0].bitangentOS),
		normalize(invTranspModel * gs_in[0].normalOS)
	);
	//TBN = gs_in[2].TBN0;
	//recalculateTBN(invTranspModel, newPos2.xyz, newPos0.xyz, newPos1.xyz, gs_in[2].texCoords, gs_in[0].texCoords, gs_in[1].texCoords);
	EmitVertex();

    EndPrimitive();
}
